@@ -121,15 +121,19 @@ sub timer {
 
    } else {
       # IO::Async has problems with overloaded objects
-      $id = $LOOP->enqueue_timer (delay => $arg{after}, code => sub { &$cb });
+      $id = $LOOP->enqueue_timer (delay => $arg{after}, code => sub {
+         undef $id; # IO::Async <= 0.43 bug workaround
+         &$cb;
+      });
    }
 
-   bless \$id, "AnyEvent::Impl::IOAsync::timer"
+   bless \\$id, "AnyEvent::Impl::IOAsync::timer"
 }
 
 sub AnyEvent::Impl::IOAsync::timer::DESTROY {
    # Need to be well-behaved during global destruction
-   $LOOP->cancel_timer (${$_[0]}) if ${$_[0]};
+   $LOOP->cancel_timer (${${$_[0]}})
+      if defined ${${$_[0]}}; # IO::Async <= 0.43 bug workaround
 }
 
 sub io {
@@ -217,7 +221,7 @@ sub _poll {
 }
 
 sub AnyEvent::CondVar::Base::_wait {
-   $LOOP->loop_once until $_[0]{_ae_sent};
+   $LOOP->loop_once until exists $_[0]{_ae_sent};
 }
 
 1;
